# +-----------------------------------------------------------------------+
# | pLoader - a Perl photo uploader for Piwigo                            |
# +-----------------------------------------------------------------------+
# | Copyright(C) 2008      Piwigo Team                  http://piwigo.org |
# +-----------------------------------------------------------------------+
# | This program is free software; you can redistribute it and/or modify  |
# | it under the terms of the GNU General Public License as published by  |
# | the Free Software Foundation                                          |
# |                                                                       |
# | This program is distributed in the hope that it will be useful, but   |
# | WITHOUT ANY WARRANTY; without even the implied warranty of            |
# | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      |
# | General Public License for more details.                              |
# |                                                                       |
# | You should have received a copy of the GNU General Public License     |
# | along with this program; if not, write to the Free Software           |
# | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, |
# | USA.                                                                  |
# +-----------------------------------------------------------------------+
package Uploader::GUI::wxApp;
use strict;
use base qw/Wx::App Class::Accessor::Fast/;    
use Wx qw/
             wxBITMAP_TYPE_GIF 
             wxBITMAP_TYPE_ICO 
             wxBITMAP_TYPE_BMP 
             wxBITMAP_TYPE_PNG 
             wxBITMAP_TYPE_JPEG
             wxIMAGE_QUALITY_NORMAL
             wxIMAGE_QUALITY_HIGH
             wxSPLASH_CENTRE_ON_SCREEN 
             wxSPLASH_TIMEOUT
             wxDefaultPosition 
             wxDefaultSize 
             wxSIMPLE_BORDER 
             wxFRAME_TOOL_WINDOW
             wxFRAME_NO_TASKBAR wxSTAY_ON_TOP
             wxWHITE
             wxICON_EXCLAMATION
             wxOK 
         /;
use Uploader::GUI::wxFrameAUI;
use File::Slurp ; 
use Data::Dumper;
use Storable;
use File::HomeDir;
use File::Spec;
use Uploader::PWG;
use Uploader::ImageList;
use Uploader::GUI::wxLoginDlg;

$|=1;

__PACKAGE__->mk_accessors( 
    qw/
          pwg
          site_url
          site_username
          site_password
          rejects
          imagelist
          storable_file
          wx_thumb_dir
          resized_dir
          preview_dir
          userdata_dir
          resized_dir
          thumb_dir
          conf_file
      / 
);

sub FilterEvent {
    my( $self, $event ) = @_;
    
    Wx::LogMessage "EventType %s", $event->GetEventType();
    
    return -1;
}

sub OnInit {
    my( $self ) = @_;

    Wx::InitAllImageHandlers();
    my $applicationName = "pLoader" ;
    $self->SetAppName( $applicationName );
    $self->SetVendorName( "Piwigo Team" );

    $self->{IMGTYPE} = {
        'jpg' => wxBITMAP_TYPE_JPEG,
        'gif' => wxBITMAP_TYPE_GIF,
        'png' => wxBITMAP_TYPE_PNG,
    };

    $self->_init_userdir;


    my $conf = retrieve $self->conf_file if -e $self->conf_file;	

    if(defined $conf ){
        $self->SetKeyValues($conf);
    }
    else {
        $self->_readParams( 'pLoader.ini' ) unless defined $conf ;
    }

    $self->site_url(
        $self->{site_url}
    );

    $self->site_username(
        $self->{site_username}
    );
    $self->site_password(
        $self->{site_password}
    );

    

    my $ok = $self->Login();
    # user pressed OK
    if($ok){
        $self->pwg(
            # get these parameters from dialog or from file
            Uploader::PWG->new(
                {
                    site_url       => $self->site_url,
                    site_username  => $self->site_username,
                    site_password  => $self->site_password,
                }
            )
        );
      
    
        $self->_init_imagelist;
        $self->_init_thumbimagelist;
        
        $self->_init_frame;
    }

    $ok;
}

sub StoreConnectionProperties {
    my ( $self ) = @_;

    eval {    
        store( 
            {
                map{
                   $_ => $self->{$_},
                }
                qw/
                    site_url
                    site_username
                    site_password
                /
            },
            $self->conf_file
        );
    };
}

sub _init_imagelist {
    my ( $self ) = @_;	

    my $stored_imagelist;
    
    if( -e $self->storable_file ){
        eval {
            $stored_imagelist = retrieve $self->storable_file;
        };
        if($@){
            Wx::LogMessage(
                "An error has occured. Can not read %s\n%s",
                $self->storable_file,
                $@
            );
            $stored_imagelist = $self->_default_imagelist_params ;
        }
    }
    else{
        $stored_imagelist = $self->_default_imagelist_params ;
    }

    $self->imagelist(
        Uploader::ImageList->new(
            $stored_imagelist
        )
    );

    $self->imagelist->ResizeCallback(
        sub { $self->ResizeImage(@_) }
    );
}


sub _default_imagelist_params {
    my ( $self ) = @_ ;

    my $params = {
       	thumb_size       => 120,
        site_thumb_dir   => $self->thumb_dir,
       	wx_thumb_size    => 120,
        wx_thumb_dir     => $self->wx_thumb_dir,
        preview_ratio    => 25, 
        preview_dir      => $self->preview_dir,
        resize_w         => 800,
        resize_h         => 600,
        site_resized_dir => $self->resized_dir,
      	type             => 'jpg',
      	filter           => 'Lanczos',
      	blur             => 0.9,
      	quality          => 100,
      	prefix           => 'TN',
      	count            => 0,
        storable_file    => $self->storable_file,
        images           => [],
        userdata_dir     => $self->userdata_dir,
        default_name_prefix => 'Photo',
        upload_rejects  =>  [],
        ResizeCallback  => sub { $self->ResizeImage(@_) },
    };

    return $params;
}

sub Login {
    my ( $self ) = @_;	

    my $dlg = Uploader::GUI::wxLoginDlg->new(
        {
            title         => "Piwigo login",
            site_url      => sub { $self->site_url(@_) },
            site_username => sub { $self->site_username(@_) },	
            site_password => sub { $self->site_password(@_) },	
        }
    );

    my $icon = Wx::Icon->new();
    $icon->LoadFile('../res/favicon.ico', wxBITMAP_TYPE_ICO);
    $dlg->SetIcon($icon);	

    
    my $rval = $dlg->ShowModal();
    $dlg->Destroy;
    
    $rval;
}

sub _init_userdir {
    my ( $self ) = @_;
    
    my $applicationName = $self->GetAppName ;
    my $userdatadir = File::Spec->canonpath(File::Spec->catfile(File::HomeDir->my_data(), "\.$applicationName"));

    mkdir $userdatadir unless -d $userdatadir ;
    $self->userdata_dir($userdatadir);
    
    $self->conf_file(
        File::Spec->catfile(
            $self->userdata_dir, 
            ".$applicationName.conf"
        )
    );

    

    $self->storable_file(
        File::Spec->catfile($self->userdata_dir, 'pLoader.dat')
    );
    
    my $thumbdir = File::Spec->catfile($self->userdata_dir, 'thumbnails');
    mkdir $thumbdir unless -d $thumbdir ;
    $self->thumb_dir($thumbdir);

    my $wxthumbdir = File::Spec->catfile($self->userdata_dir, 'wxthumbnails');
    mkdir $wxthumbdir unless -d $wxthumbdir ;
    $self->wx_thumb_dir($wxthumbdir);


    my $resizedir = File::Spec->catfile($self->userdata_dir, 'resize');
    mkdir $resizedir unless -d $resizedir ;
    $self->resized_dir($resizedir);

    my $previewdir = File::Spec->catfile($self->userdata_dir, 'preview');
    mkdir $previewdir unless -d $previewdir ;
    $self->preview_dir($previewdir);

	
}

sub _init_thumbimagelist {
    my ( $self ) = @_;

    
    $self->imagelist->wx_thumb_imglist(
        Wx::ImageList->new( 
            $self->imagelist->wx_thumb_size, 
            $self->imagelist->wx_thumb_size, 
            1,
            0
        )
    );
    
    # reload images 
    $self->_reload_thumb_images;
}


sub _reload_thumb_images {
    my ( $self ) = @_;
    
    my $wximagelist = $self->imagelist->wx_thumb_imglist;
    my $images = $self->imagelist->images;

    map {
        $wximagelist->Add(
            Wx::Bitmap->new( 
                $_->wx_thumb_file, 
                $self->GetWxBitmapType($self->imagelist->type), 
            )
        );
    }
    @$images ;

}

sub GetWxBitmapType {
    my ( $self, $type ) = @_;
    
    $self->{IMGTYPE}->{$type};
}


sub ResizeImage {
    my ( $self, $image_file, $image_file_out, $type, $ratio, $width, $height, $quality ) = @_;
   

    my $image = Wx::Image->new(
            $image_file, 
            $self->GetWxBitmapType($type),
            0
    );
    
    my $w;
    my $h;

    my $img_w = $image->GetWidth;
    my $img_h = $image->GetHeight;
    
    # use a ratio ( 25% default ) if defined
    # default ratio is used for preview.
    if($ratio){
    	$w = $ratio*$img_w/100 ;
        $h = $ratio*$img_h/100 ;
    }
    # use specified width and height
    else{
        # portrait
        if( $img_w < $img_h ){
            $w = $height;
        }
        else{
            $w = $width;
        }
        # to respect aspect ratio
        $h = sprintf(
            "%.0f",
            ($w*$img_h)/$img_w
        );
    }

    $image->Rescale(
        $w,
        $h,
        wxIMAGE_QUALITY_HIGH
    );
    
    $quality ||= 90;
    
    $image->SetOption( 
        "quality", 
        $quality 
    );
    
    if(!$image->SaveFile(
        $image_file_out,
        $self->GetWxBitmapType($type),
    )){
    	Wx::LogMessage(
    	    "An error has occured. Can not save file %s",
    	    $image_file_out,
    	)
    };
}

sub _init_frame {
    my ( $self ) = @_ ;	

    my $config = Wx::ConfigBase::Get;

    my $x = $config->ReadInt( "X", 50 );
    my $y = $config->ReadInt( "Y", 50 );
    my $w = $config->ReadInt( "Width", 1024 );
    my $h = $config->ReadInt( "height", 768 );
    my $sv = $config->ReadInt( "splitVertical", 150 );
    my $sh = $config->ReadInt( "splitHorizontal", 300 );
	
    my $url;
    if($self->pwg->login_result->{stat} eq 'ok'){
        $url = $self->site_url;    
    }
    else{
        $url = "Not connected";
        Wx::MessageBox( 
            sprintf(
                "%s\n\nConnection to %s failed",
                $self->pwg->login_result->{message},
                $self->site_url,
            ),
            "Piwigo login error",
            wxOK | wxICON_EXCLAMATION, 
        );
    }	
    	

    my ( $frame ) = Uploader::GUI::wxFrameAUI->new( 
        {
            x         => $x,
            y         => $y,
            width     => $w,
            height    => $h,
            title     => "pLoader - Piwigo uploader 1.0 RC2 - [$url]" ,
            pwg       => $self->pwg,
            imagelist => $self->imagelist,
        }
    );
	
	#printf("Config x %s, y %s, w %s, h %s\n", $x, $y, $w, $h);
	#$frame->SetSize( $x, $y, $w, $h );
    $frame->Show( 1 );
    $self->SetTopWindow( $frame );

    my $icon = Wx::Icon->new();
    $icon->LoadFile('../res/favicon.ico', wxBITMAP_TYPE_ICO);
    $frame->SetIcon($icon);	


    return 1;
}


sub _readParams {
	my( $self, $file ) = @_ ;


	my $expr_params ;
	eval { $expr_params = read_file( $file ); } ;
	
	my $paramValues = [] ;
	if($expr_params){
		my $expr = '$paramValues = ' ;
		$expr .=  "$expr_params ; " ;
		eval $expr ;
	}
	
	return unless 'ARRAY' eq ref $paramValues ;
	
	if(scalar(@$paramValues )){
	    my $params = $paramValues->[0] ;
	    $self->SetKeyValues($params);
	}
}

sub SetKeyValues {
    my ( $self, $params )= @_;	

    foreach( keys %$params ) {
        $self->{$_} = $params->{$_} ;
    }
}

1;