# +-----------------------------------------------------------------------+
# | pLoader - a Perl photo uploader for Piwigo                            |
# +-----------------------------------------------------------------------+
# | Copyright(C) 2008      Piwigo Team                  http://piwigo.org |
# +-----------------------------------------------------------------------+
# | This program is free software; you can redistribute it and/or modify  |
# | it under the terms of the GNU General Public License as published by  |
# | the Free Software Foundation                                          |
# |                                                                       |
# | This program is distributed in the hope that it will be useful, but   |
# | WITHOUT ANY WARRANTY; without even the implied warranty of            |
# | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      |
# | General Public License for more details.                              |
# |                                                                       |
# | You should have received a copy of the GNU General Public License     |
# | along with this program; if not, write to the Free Software           |
# | Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, |
# | USA.                                                                  |
# +-----------------------------------------------------------------------+
package Uploader::GUI::wxFrameAUI;
use strict;
use Wx;
use Wx::DND;
use Wx qw/
             wxDEFAULT_FRAME_STYLE
             wxVERTICAL
             wxGROW
             wxBITMAP_TYPE_JPEG
             wxBITMAP_TYPE_GIF
             wxBITMAP_TYPE_PNG
             wxTB_FLAT
             wxSIZE
             wxWHITE
             wxBLACK
             wxID_CANCEL
             wxFD_OPEN
             wxFD_MULTIPLE
             wxLI_HORIZONTAL
             wxALIGN_CENTER_VERTICAL
             wxALL
             wxDefaultPosition
             wxDefaultSize
             wxTheApp
             wxIMAGE_LIST_NORMAL
             wxAUI_NB_TAB_MOVE
             wxAUI_NB_TAB_SPLIT
             wxNO_BORDER
             wxTE_MULTILINE
             wxTE_READONLY
             wxITEM_NORMAL 
             wxCLIP_CHILDREN
             wxBORDER_NONE
             wxNullBitmap 
             wxTR_MULTIPLE
             wxTR_EXTENDED
             wxTR_HIDE_ROOT
             wxTR_HAS_BUTTONS
             wxMAXIMIZE
             wxOK
             wxICON_EXCLAMATION 
         /;
use base qw/Wx::Frame Class::Accessor::Fast/;
use File::Spec;

my @properties = 
    qw/
          progressdlg
          upload_progressdlg
          imageviewer
          tree
          tree_root
          treeimglist
          tree_item_default
          topsizer
          pwg
          manager
          logwnd
          oldlogwnd
          htmlhome
          notebook
          categories
          imagelist
          image_prop_general
          image_prop_exif
          notebook_image_properties
          global_settings_panel
      /;
__PACKAGE__->mk_accessors( @properties );

use Wx::Event 
    qw/
          EVT_MENU
          EVT_TREE_SEL_CHANGED
          EVT_CLOSE 
          EVT_LIST_END_LABEL_EDIT
          EVT_LIST_ITEM_SELECTED
          EVT_TREE_ITEM_RIGHT_CLICK       
      /;
      
use Uploader::GUI::wxImageListCtrl;
use Uploader::GUI::wxImageProcessingProgressDlg;
use Wx::Html;
use Uploader::GUI::wxHtmlWindow;
use Data::Dumper;
use Wx::AUI;
use Storable;
use Uploader::GUI::wxPhotoPropertiesPanel;
use Uploader::GUI::wxPhotoExifPanel;
use Uploader::GUI::wxGlobalSettingsPanel;
use Encode qw/encode decode is_utf8/;

$|=1;

my $ID_TREE_CTX_MENU = 20000 ;

sub new {
  my( $class, $params ) = @_;
  my( $self ) = $class->SUPER::new( 
                                      undef, 
                                      -1, 
                                      $params->{title}, 
                                      [ $params->{x}, $params->{y} ],
                                      [ $params->{width}, $params->{height} ],
                                      wxDEFAULT_FRAME_STYLE|wxMAXIMIZE
                                   );
    $self->pwg( $params->{pwg} );
    $self->imagelist( $params->{imagelist} );

    # callback for GUI refresh : add thumbnail images to the imageviewer control
    $self->imagelist->SetNewFilesViewerRefreshCallback(
        sub { $self->SetNewFilesViewerRefresh(@_) }
    );

    # callback for GUI refresh : progress dialog display of thumbnail image being created
    $self->imagelist->SetNewFilesProgressCallback(
        sub { $self->SetNewFilesProgress(@_) }
    );

    # callback for GUI refresh : remove thumbnail images from imageviewer control
    $self->imagelist->UploadImagesViewerCallback(
        sub { $self->UploadImagesViewerRefresh(@_) }
    );

    # callback for GUI refresh : progress dialog display current uploaded image 
    $self->imagelist->UploadImagesProgressCallback(
        sub { $self->UploadImagesProgress(@_) }
    );


    $self->imagelist->pwg(
        $self->pwg
    );

    $self->imagelist->categories(
        []
    );

    $self->manager( Wx::AuiManager->new );
    $self->manager->SetManagedWindow( $self );

    $self->_initFrame;
    $self->_InitEventHandlers;
    $self->_initImgTypes;   
   
    $self;
}

sub _initImgTypes {
    my ( $self ) = @_;

    $self->{IMGTYPE} = {
             'jpg' => wxBITMAP_TYPE_JPEG,
             'gif' => wxBITMAP_TYPE_GIF,
             'png' => wxBITMAP_TYPE_PNG,
    };
}


sub GetWxBitmapType {
    my ( $self, $type ) = @_;
    
    $self->{IMGTYPE}->{$type};
}


sub _initFrame {
    my ( $self ) = @_;
    
    $self->CreateStatusBar(1);

    $self->htmlhome(
        Uploader::GUI::wxHtmlWindow->new(
            $self, -1
        )
    );
    $self->imageviewer(
        Uploader::GUI::wxImageListCtrl->new( 
            { 
                parentwnd => $self,
                imagelist => $self->imagelist,
            }
        )
    );
    
    $self->global_settings_panel(
        $self->create_global_settings_panel()
    );    
    
    $self->image_prop_general(
        Uploader::GUI::wxPhotoPropertiesPanel->new( 
            { 
                parentwnd => $self,
                accessors => {
                	          $main::ID_PHOTO_NAME =>
                	              sub { $self->imagelist->current_image->site_name(@_) }, 

                	          $main::ID_PHOTO_CREATE_DATE =>
                	              sub { $self->imagelist->current_image->create_date(@_) }, 
                                  
                                  $main::ID_PHOTO_COMMENT =>
                	              sub { $self->imagelist->current_image->site_comment(@_) }, 
                                  
                                  $main::ID_PHOTO_AUTHOR =>
                	              sub { $self->imagelist->current_image->site_author(@_) }, 
                             },
                frame_callbacks => { 
                  	              $main::ID_PHOTO_NAME =>
                                          sub {
                                              $self->imageviewer->Refresh();
                                          },
                	           },
            }
        )
    );

    $self->image_prop_exif(
        Uploader::GUI::wxPhotoExifPanel->new( 
            { 
                parentwnd => $self,
                frame_callback => sub { 
                                      $self->imageviewer->Refresh();
                	          },
                accessors => {
                	          $main::ID_EXIF_CREATE_DATE =>
                	              sub { $self->imagelist->current_image->create_date }, 
                	          $main::ID_EXIF_MODEL =>
                	              sub { $self->imagelist->current_image->exif_tag('Model') }, 
                	          $main::ID_EXIF_WIDTH =>
                	              sub { $self->imagelist->current_image->exif_tag('ImageWidth') }, 
                	          $main::ID_EXIF_HEIGHT =>
                	              sub { $self->imagelist->current_image->exif_tag('ImageHeight') }, 
                	          $main::ID_EXIF_ORIENTATION =>
                	              sub { $self->imagelist->current_image->exif_tag('Orientation') }, 
                	          $main::ID_EXIF_ISO =>
                	              sub { $self->imagelist->current_image->exif_tag('ISO') }, 
                	          $main::ID_EXIF_SHUTTER_SPEED =>
                	              sub { $self->imagelist->current_image->exif_tag('ExposureTime') }, 
                	          $main::ID_EXIF_APERTURE =>
                	              sub { $self->imagelist->current_image->exif_tag('ApertureValue') }, 
                	          $main::ID_EXIF_FOCAL_LENGTH =>
                	              sub { $self->imagelist->current_image->exif_tag('FocalLength') }, 
                	          $main::ID_EXIF_LENS =>
                	              sub { $self->imagelist->current_image->exif_tag('Lens') }, 
                             },
            }
        )
    );


    $self->imageviewer->Show(0);

    $self->imageviewer->SetDropTarget( 
        DNDImageListDropTarget->new(
            $self->imageviewer 
        ) 
    );
    $self->htmlhome->SetDropTarget( 
        DNDImageListDropTarget->new( 
            $self->imageviewer 
        ) 
    );

    $self->manager->AddPane
      ( $self->create_notebook_imagelist, Wx::AuiPaneInfo->new->Name( "ImageViewer" )
        ->CenterPane->Resizable->CloseButton(0) );


    $self->manager->AddPane
      ( $self->create_tree, Wx::AuiPaneInfo->new->Name( "categories" )->Caption(
          sprintf("Piwigo categories - %s", wxTheApp->site_url )
      )
        ->Right->Position( 0 )->Resizable->CloseButton(0) );

    $self->tree->SetDropTarget( 
        DNDCategoryTreeDropTarget->new(
            $self->tree
        )
    );


    $self->manager->AddPane
      ( $self->create_notebook_image_properties, Wx::AuiPaneInfo->new->Name( "photo_properties" )->Caption("Photo properties")
        ->Right->Position( 1 )->Resizable->CloseButton(0) );

    $self->manager->AddPane
      ( $self->create_notebook_global_settings, Wx::AuiPaneInfo->new->Name( "global_settings" )
        ->Right->Position( 2 )->Resizable->CloseButton(1) );



    my $tb = $self->create_toolbar;

    $self->manager->AddPane
      ( $tb, Wx::AuiPaneInfo->new->Name( "tb1" )->Caption( "Tools" )
        ->ToolbarPane->Top->Row( 1 )->LeftDockable( 0 )->RightDockable( 0 ) ) if defined $tb;

    $self->manager->Update;

    if( $self->imagelist->wx_thumb_imglist->GetImageCount){
    	$self->ShowImageViewer;
        
        $self->imageviewer->Refresh(
            $self->imagelist->wx_thumb_imglist
        );
    }

}


sub OnGlobalSettings {
    my ( $self ) = @_;

    $self->manager->GetPane("global_settings")->Show(1);
    $self->manager->Update();    
}


sub create_notebook_imagelist {
    my( $self ) = @_;
    my $book = Wx::AuiNotebook->new( $self, -1, [-1, -1], [300, 450],
                                     wxBORDER_NONE|wxAUI_NB_TAB_MOVE|wxAUI_NB_TAB_SPLIT);
    my $pages = [    
        [
            $self->htmlhome, 
            "Photo selection",
            '../res/images.png', 
            wxBITMAP_TYPE_PNG,
        ],
        [
            $self->logwnd( $self->_create_textctrl( $book, undef, [450, 450] )),
            "Log",
            '../res/nb_log.png', 
            wxBITMAP_TYPE_PNG,
        ]
    ];
    
    $self->_populate_notebook( $book, $pages );

    $self->logwnd(Wx::LogTextCtrl->new( $self->logwnd ));
    $self->oldlogwnd( Wx::Log::SetActiveTarget( $self->logwnd ) );
    $self->notebook($book);
    return $book;
}

sub create_notebook_image_properties {
    my( $self ) = @_;
    my $book = Wx::AuiNotebook->new( $self, -1, [-1, -1], [450, 450],
                                     wxBORDER_NONE|wxAUI_NB_TAB_MOVE|wxAUI_NB_TAB_SPLIT);
    my $pages = [    
        [
            $self->image_prop_general, 
            "Piwigo",
        ],
        [
            $self->image_prop_exif, 
            "Exif",
        ],
    ];
    
    $self->_populate_notebook( $book, $pages );

    $self->notebook_image_properties($book);

    return $book;
}

sub create_notebook_global_settings {
    my( $self ) = @_;
    my $book = Wx::AuiNotebook->new( $self, -1, [-1, -1], [450, 450],
                                     wxBORDER_NONE|wxAUI_NB_TAB_MOVE|wxAUI_NB_TAB_SPLIT);
    my $pages = [    
        [
            $self->global_settings_panel, 
            "Global settings",
        ],
    ];
    
    $self->_populate_notebook( $book, $pages );

    $self->notebook_image_properties($book);

    return $book;
}

sub create_global_settings_panel {
    my ( $self ) = @_;

    Uploader::GUI::wxGlobalSettingsPanel->new( 
            { 
                parentwnd => $self,
                accessors => {
                	          $main::ID_GLOBAL_NAME_PREFIX =>
                	              sub { $self->imagelist->default_name_prefix(@_) }, 
                	          $main::ID_GLOBAL_AUTHOR =>
                	              sub { $self->imagelist->author(@_) }, 
                	          $main::ID_GLOBAL_THUMB_SIZE => 
                	              sub { $self->imagelist->thumb_size(@_) }, 
                	          $main::ID_GLOBAL_RESIZED_W =>
                	              sub { $self->imagelist->resize_w(@_) }, 
                	          $main::ID_GLOBAL_RESIZED_H =>
                	              sub { $self->imagelist->resize_h(@_) }, 
                             },
                frame_callbacks => { 
                	           },
            }
        );
}

sub create_tree {
    my ( $self ) = @_;

    my $images = [
        map {
            Wx::Bitmap->new( $_, wxBITMAP_TYPE_PNG )
        	
        }
        (
          '../res/tree_pwg.png',
          '../res/tree_folder.png',
        )    
    ];

    $self->treeimglist( Wx::ImageList->new( 16, 16, 1 ) );
    map {
    	$self->treeimglist->Add($_);
    }
    @$images;

    
    $self->tree( 
        Wx::TreeCtrl->new( 
           $self, 
           -1, 
           wxDefaultPosition, 
           wxDefaultSize, 
           wxBORDER_NONE|
           wxCLIP_CHILDREN|
#           wxTR_MULTIPLE|
#           wxTR_EXTENDED|
           wxTR_HAS_BUTTONS
#           wxTR_HIDE_ROOT   
        ) 
    );

    $self->tree->SetImageList( $self->treeimglist );
    $self->populate_tree_categories;
    $self->tree ;
}


sub populate_tree_categories {
    my ( $self ) = @_;

    $self->populate_tree(
        $self->tree,
        wxTheApp->pwg->categories,
    ) if defined wxTheApp->pwg ;
    
    $self->tree->ExpandAll;

}

#$tree_items is a list of anonymous arrays
# [ itemLabel, undef, rootInfo, ImageIndex, SelImageIndex ] for the root

# [ itemLabel, 
#       [ childItemLabel, 
#           [...], 
#           childItemdata, 
#           childImageIndex, 
#           childSelImageIndex
#       ], 
#    Itemdata, 
#    ImageIndex, 
#    SelImageIndex 
# ]

# returns a valid itemData
sub itemData { Wx::TreeItemData->new( $_[0] ) }

sub populate_tree {
    my ( $self, $tree, $tree_items ) = @_;
    my $root = shift @{$tree_items};
    $self->tree_root(
        $tree->AddRoot( 
            $root->[0], 
            $root->[3], 
            $root->[4], 
            itemData( $root->[2] ) 
        )
    );

  $self->populate_tree_helper( $tree, $self->tree_root, $tree_items );

  $tree->Expand( $self->tree_root );
}

sub populate_tree_helper {
  my ( $self, $tree, $parent_id, $tree_items ) = @_;

  my $id;

  map {
      my $name = $_->[0];
      $id = $tree->AppendItem( 
                                  $parent_id, 
                                  is_utf8($name) ? decode('UTF-8', $name) : $name,
                                  defined($_->[3]) ? $_->[3] : 0, 
                                  defined($_->[4]) ? $_->[4] : 0, 
                                  itemData( $_->[2]) 
                             );
      $self->tree_item_default($id) if ! defined $self->tree_item_default;
      # current item has children
      if( ref( $_->[1] ) eq 'ARRAY' ) {
          $self->populate_tree_helper( $tree, $id, $_->[1] );
      } 
  }
  @{$tree_items};
  

}




sub _populate_notebook {
    my ( $self, $book, $pages, $position ) = @_;	

    my $i = 0;
    map {
        my $icon = Wx::Icon->new();
        eval { $icon->LoadFile($_->[2], $_->[3]) ; }
            if defined $_->[2];
        my $nb_icon = Wx::Bitmap->new( $icon );
        defined $position ?
            $book->InsertPage($position, $_->[0],
                    $_->[1], 1, $nb_icon )
                          :
            $book->AddPage($_->[0],
                    $_->[1], 0, $nb_icon );

        $i++;
    } @$pages;

}



sub _InitEventHandlers {
    my ( $self ) = @_ ;
	
    Wx::Event::EVT_MENU( $self, 101, \&OnAddImages );
    Wx::Event::EVT_MENU( $self, 102, \&OnRemoveImages );
    Wx::Event::EVT_MENU( $self, 103, \&OnUploadImages );
    Wx::Event::EVT_MENU( $self, 104, \&OnGlobalSettings );
    EVT_TREE_SEL_CHANGED( $self, $self->tree, \&OnTreeSelChanged );
    EVT_TREE_ITEM_RIGHT_CLICK( $self->tree, $self->tree, \&OnTreeItemRightClick );
    EVT_LIST_END_LABEL_EDIT( $self, $self->imageviewer, \&OnImageViewerEndLabelEdit );
    EVT_LIST_ITEM_SELECTED($self, $self->imageviewer, \&OnImageViewerItemSelected) ;
    EVT_CLOSE( $self, \&OnClose );


    Wx::Event::EVT_MENU( $self, 1+$ID_TREE_CTX_MENU, \&OnAddCategories );
    Wx::Event::EVT_MENU( $self, 2+$ID_TREE_CTX_MENU, \&OnRefreshCategories );
}

{
  my $prevdir;
  my $prevfile;

  sub OnAddImages {
    my( $self, $event ) = @_;
    my $dialog = Wx::FileDialog->new
      ( $self, "Select photos to send", $prevfile, $prevdir,
        "JPEG files (*.jpg)|*.jpg|All(*.*)|*.*",
        wxFD_OPEN|wxFD_MULTIPLE );

    my $file_paths = [];
    if( $dialog->ShowModal != wxID_CANCEL ) {
        @$file_paths = $dialog->GetPaths;
        $self->SetNewFiles($file_paths) ;
    }
    $dialog->Destroy;
    $event->Skip;
  }
}

sub OnRemoveImages {
    my( $self, $event ) = @_;

    
    $self->imagelist->RemoveImageSelection;
    $self->imageviewer->Refresh;	
}

sub SetNewFiles {
    my ( $self, $file_paths ) = @_;

    $self->ShowImageViewer();
    $self->progressdlg( 
        Uploader::GUI::wxImageProcessingProgressDlg->new(
            { 
                title => "Image processing progress information", 
             }
        )        
    );
    $self->progressdlg->Show(1);
    Wx::Yield();

    $self->imagelist->SetNewFiles(
        $file_paths
    );

    
}

sub OnTreeSelChanged {
    my( $self, $event ) = @_;
  
    my @items = $self->tree->GetSelections;

    $self->imagelist->categories(
        [
            map {
                my $category = $self->tree->GetPlData( $_ );
                $category->{id} if $category != -1;
            }
            @items
        ]
    );
}

sub OnTreeItemRightClick {
    my( $self, $event ) = @_;

    my $point = $event->GetPoint ;
    my $ctx_mnu = Wx::Menu->new;
    
    map {
    	$ctx_mnu->Append(
    	    @$_
    	);
    }
    (
        [1+$ID_TREE_CTX_MENU, "Add new category","Add a new category to the current selected category."],
        [2+$ID_TREE_CTX_MENU, "Refresh","Refresh category list."],
    );
    $self->PopupMenu($ctx_mnu, $point);
	
}

sub OnAddCategories {
    my ( $self, $event ) = @_;

    my $item = $self->tree->GetSelection;

    my $category = $self->tree->GetPlData($item);
    my $category_id;
    
    $category_id = $category->{id} if 'HASH' eq ref($category) ;

    my $dialog = Wx::TextEntryDialog->new( 
        $self, 
        "Category name :", 
        "Add new category",
        "New category" 
    );

    if( $dialog->ShowModal != wxID_CANCEL ) {
    	my $name = $dialog->GetValue;
        my ( $status, $status_msg ) = $self->pwg->AddCategories( $name, $category_id);
        $self->_refresh_tree_categories_helper;
    }
    $dialog->Destroy;
}

sub OnRefreshCategories {
    my ( $self, $event ) = @_;

    $self->_refresh_tree_categories_helper;
}

sub _refresh_tree_categories_helper {
    my ( $self ) = @_;	

    my $busycursor = Wx::BusyCursor->new();
    $self->tree->CollapseAll;
    $self->tree->DeleteAllItems;
    $self->imagelist->categories([]);
    $self->pwg->RefreshCategories();
    $self->populate_tree_categories;
}

sub OnImageViewerEndLabelEdit {
    my( $self, $event ) = @_;
  
    my $id = $event->GetItem;

    my $image = $self->imagelist->images->[$event->GetIndex];
    $image->site_name(
        $event->GetLabel
    );
    	
    $self->image_prop_general->SetControlValues();
}


sub OnImageViewerItemSelected {
    my( $self, $event ) = @_;
  
    my $indx = $event->GetIndex;

    # retrieve photo info for the last selected photo
    $self->imagelist->SetCurrentImage($indx);
    
    $self->imagelist->image_selection(
        $self->imageviewer->GetSelectedItems
    );

    $self->image_prop_general->SetControlValues();
    $self->image_prop_exif->SetControlValues();
}

sub OnUploadImages {
    my( $self, $event ) = @_;

    
    $self->ProcessImageSelection();    
}

# remove image from imagelist when uploaded
sub UploadImagesViewerRefresh {
    my ( $self ) = @_;	

    my $imagelist= $self->imagelist ;

    Wx::Yield();
}

# display uploaded image in progress dlg
sub UploadImagesProgress {
    my ( $self ) = @_;	

    my $imagelist = $self->imagelist ;

    my ( $vol, $dir, $file ) = File::Spec->splitpath($imagelist->current_image->file);

    $self->upload_progressdlg->processing(
        sprintf(
            $imagelist->progress_msg, 
            $imagelist->current_image->site_name,
            $file,
        )
    );
    eval {
        $self->upload_progressdlg->image->SetBitmap(wxNullBitmap);
        $self->upload_progressdlg->image->SetBitmap(
            Wx::Bitmap->new( 
                $self->imagelist->current_image->wx_thumb_file, 
                $self->GetWxBitmapType($self->imagelist->type), 
            )
        );
        $self->upload_progressdlg->progress(
            $imagelist->count * ( 100/scalar @{$imagelist->image_selection} )
        );
        $self->upload_progressdlg->LogProgress();
    };
    Wx::Yield();

}

sub ShowImageViewer {
    my ( $self ) = @_;	

    if(!$self->imageviewer->IsShown){
        my $imglistpage = [
            [
                $self->imageviewer, 
                "Photo selection",
                '../res/images.png', 
                wxBITMAP_TYPE_PNG,
            ],
        ];
        $self->notebook->DeletePage(0);
    
        $self->imageviewer->Show(1);
        $self->_populate_notebook($self->notebook, $imglistpage, 0);
        # load imagelist with loading.jpg
    }
	
}




sub SetNewFilesViewerRefresh {

    my ( $self ) = @_;	

    my $wximagelist = $self->imagelist->wx_thumb_imglist;

    my $indx = $wximagelist->Add(
        Wx::Bitmap->new( 
            $self->imagelist->current_image->wx_thumb_file, 
            $self->GetWxBitmapType($self->imagelist->type), 
        )
    ) if defined $self->imagelist->current_image->wx_thumb_file;
    
    $self->imageviewer->Refresh(
        $wximagelist
    );

    Wx::Yield();
}



# prepare and upload image_selection
sub ProcessImageSelection {
    my ( $self ) = @_;

    return if !scalar @{$self->imagelist->images};

    if( scalar @{$self->imagelist->categories} ){
        return if( !defined $self->imagelist->image_selection );
        return if( !scalar @{$self->imagelist->image_selection} );
        
        $self->upload_progressdlg(
            Uploader::GUI::wxImageProcessingProgressDlg->new(
                { 
                    title => "Image upload progress information", 
                 }
            )        
        );
        $self->upload_progressdlg->Show(1);
        Wx::Yield();
        $self->imagelist->UploadSelection;
    }
    else {
        Wx::MessageBox( 
            sprintf(
                "Please select a valid target category",
            ),
            "Piwigo upload error",
            wxOK | wxICON_EXCLAMATION, 
        );
    }
}


sub SetNewFilesProgress {
    my ( $self ) = @_;

    my $imagelist = $self->imagelist;

    $self->progressdlg->processing(
        sprintf(
            $imagelist->progress_msg, 
            $imagelist->current_image->file,
        )
    );

    eval {
        $self->progressdlg->image->SetSize([ $imagelist->wx_thumb_size, $imagelist->wx_thumb_size]);
        $self->progressdlg->image->SetBitmap(wxNullBitmap);
        $self->progressdlg->image->SetBitmap(
            Wx::Bitmap->new( 
                $imagelist->current_image->wx_thumb_file,
                $self->GetWxBitmapType( $imagelist->type )
            )
        );
        $self->progressdlg->progress(
            $imagelist->count * ( 100/scalar @{$imagelist->new_files} )
        );
        $self->progressdlg->LogProgress();
    };
    Wx::Yield();
}

sub OnClose {
  my $self = shift;

    # Restaure previous log wnd
    Wx::Log::SetActiveTarget( $self->oldlogwnd );

    # allways store
    
    wxTheApp->StoreConnectionProperties;
    
    $self->imagelist->Store;
    $self->Destroy;
}


sub create_toolbar {
    my( $self ) = @_;

    my $tb = Wx::ToolBar->new( $self, -1, wxDefaultPosition, wxDefaultSize, wxTB_FLAT );
    $tb->SetToolBitmapSize( wxSIZE( 32, 32 ) );
    map {
        my $icon1 = Wx::Icon->new();
        eval {
            $icon1->LoadFile($_->[2], $_->[3]);
        };
        my $tb_icon1 = Wx::Bitmap->new( $icon1 );

        my $icon2 = Wx::Icon->new();
        eval {
            $icon2->LoadFile($_->[5], $_->[3]);
        };
        my $tb_icon2 = Wx::Bitmap->new( $icon2 );


        $tb->AddTool( $_->[0], $_->[1], $tb_icon1, $tb_icon2, wxITEM_NORMAL, $_->[1] );
        $tb->EnableTool( $_->[0], $_->[4]);
    }
    (
        [
            101, "Add photo to selection", '../res/tb_add.png', wxBITMAP_TYPE_PNG, 1, '../res/tb_add.png', "Add photo to selection for resizing and uploading"
        ],
        [
            102, "Remove photo from selection", '../res/tb_remove.png', wxBITMAP_TYPE_PNG, 1, '../res/tb_remove.png',"Remove photo from selection. Files are not deleted "
        ],
        [
            103, "Upload to Piwigo", '../res/tb_upload.png', wxBITMAP_TYPE_PNG, 1, '../res/tb_upload.png',"Upload photos to Piwigo."
        ],
        [
            104, "Global settings", '../res/tb_settings.png', wxBITMAP_TYPE_PNG, 1, '../res/tb_settings.png',"Change global settings."
        ],
    
    );

    if($@){
    	undef $tb;
    }
    else{
        $tb->Realize;
    }

    return $tb;
}


sub _create_textctrl {
    my( $self, $parent, $text, $size ) = @_;

    return Wx::TextCtrl->new( $parent, -1, $text, [0, 0], $size,
                              wxNO_BORDER | wxTE_MULTILINE | wxTE_READONLY  );
}

sub create_textctrl {
    my( $self, $text, $size ) = @_;

    return $self->_create_textctrl( $self, $text, $size );
}

sub DESTROY {
    my( $self ) = @_;

    $self->manager->UnInit;
}



1;





package DNDImageListDropTarget;

use base qw(Wx::FileDropTarget Class::Accessor::Fast);

__PACKAGE__->mk_accessors( 
    qw/
          imageviewer
          frame
      /
);

sub new {
  my $class = shift;
  my $imageviewer = shift;
  my $self = $class->SUPER::new( @_ );

  $self->imageviewer($imageviewer);
  $self->frame($imageviewer->GetParent);
  return $self;
}

sub OnDropFiles {
  my( $self, $x, $y, $files ) = @_;

  $self->frame->SetNewFiles($files) ;
}

1;


package DNDCategoryTreeDropTarget;

use base qw(Wx::TextDropTarget Class::Accessor::Fast);
use Data::Dumper;
use Wx qw/
              wxDragNone
              wxDragCopy
              wxDragMove
         /;

__PACKAGE__->mk_accessors( 
    qw/
          tree
          frame
      /
);

sub new {
  my ( $class, $tree ) = @_;
  my $self = $class->SUPER::new();

  $self->tree($tree);
  $self->frame($tree->GetParent);
  return $self;
}



sub OnDropText {
  my( $self, $x, $y, $textdata ) = @_;

  # must be $VAR1 because $textdata is the result of Data::Dumper
  my $VAR1;
  eval $textdata;
  eval {
      if(scalar @$VAR1){
      	   my @items;
      	   if(scalar @items < 2) {
               my ($dropItem, $flag) = $self->tree->HitTest([$x, $y]);
      	   	push @items, $dropItem;
      	   }
      	   else {
      	   	@items = $self->tree->GetSelections;
      	   }
      	   
      	   # remove root item which is not a valid category
      	   @items = grep { $self->tree->GetPlData( $_ ) != -1 } @items;
      	   
           $self->frame->imagelist->categories(
               [
                   map {
                       $self->tree->GetPlData( $_ )->{id};
                   }
                   @items
               ]
           );
           
      	   $self->frame->imagelist->image_selection($VAR1);
      	   $self->frame->ProcessImageSelection ;
      }	
  };
}

1;